c
c    Non-linear least squares solution of normal equations
c  For description see A.Schwarzenberg-Czernyi, 1995, A&ASupp 110,405
c  (or my publication 15)
c
      integer n1,idf,i,maxn1,mxiter,isav
      parameter (maxn1=21,mxiter=21)
      real eq(maxn1,maxn1),cracow,det,test,tol,cond
      double precision x(maxn1),xo(3,maxn1)
      data tol/1e-10/
      iter=0
      isav=0
      write(*,*) 'iter,test,sig,cond,det'
 10   continue
        iter=iter+1
        isav=mod(isav,3)+1
        call normeq(iter,x,eq,n1,maxn1,idf)
        n=n1-1
        cond=cracow(eq,n1,maxn1,idf,det)
        if (cond.le.0.) then
          write(*,*) 'Error or singular normal equations'
          do 4 i=1,n
            if (eq(i,i).eq.0.) 
     $    write(*,'(i3,a)') i,'-th column linearly dependent'
 4        continue
        endif
          test=0.
          do 3 i=1,n
            test=test+eq(i,n1)*eq(i,n1)
            x(i)=dble(eq(i,n1))+x(i)
 3          xo(isav,i)=x(i)
          test=sqrt(test)
          xo(isav,n1)=test
          if (test.ge.tol.and.iter.lt.mxiter) then
            if (mod(iter,3).eq.2) 
     $          write(*,*) iter,test,sqrt(eq(n1,n1)),cond,det
            test=xo(2,n1)*xo(2,n1)
            if (isav.eq.3 .and. 
     $         abs(xo(1,n1)*xo(3,n1)-test).lt.0.1*test) then
              isav=0
              do 7 i=1,n
                test=x(i)-xo(2,i)
 7              x(i)=x(i)-test*test/(x(i)-2.*xo(2,i)+xo(1,i)) 
            endif
            goto 10
          endif
      write(*,*) iter,test,sqrt(eq(n1,n1)),cond,det
      write(*,*) 'Covariance \ Triangle root matrices'
      do 1 i=1,n
 1      write(*,*) (eq(j,i),j=1,n1)
      write(*,*) 'Unknowns'
      write(*,*) (x(i),i=1,n)
      write(*,*) 'Errors, Std. Observ. Error'
 99   write(*,*) (sqrt(eq(i,i)),i=1,n1)
      end
c
c     NORMAL EQUATIONS BUILDER: Hilbert matrix
c
      subroutine normeq(iter,x,eq,n1,maxn1,idf)
      integer maxn1,n1,i,j,idf
      real eq(maxn1,maxn1)
      double precision x(maxn1),xx,sav,rest
c  Set initial values
      n1=8
      idf=1
      n=n1-1
      if (iter.le.1) then
        do 3 i=1,n
 3        x(i)=1D0
      endif
c  Set normal equations
      rest=0D0
      do 1 i=1,n
        xx=1D0
        sav=0D0
        do 2 j=1,n
          if (i.gt.0.and.j.gt.0) then
            eq(i,j)=1./(i+j-1)
          else
            eq(i,j)=0.
          endif
          sav=(xx-x(j))*eq(i,j)+sav
 2        xx=-(xx+sign(1d0,xx))
        eq(i,n1)=sav
        eq(n1,i)=sav        
 1      rest=rest+sav*sav
      eq(n1,n1)=rest+1D0
      end
c
c     NORMAL EQUATIONS BUILDER: Center of circle
c
      subroutine normeq1(iter,x,eq,n1,maxn1,idf)
      integer n1,n,i,j,l,idf,m1,m2
      parameter (m2=12,m1=6,n10=4)
      real eq(maxn1,maxn1),y0(m2),y(m2),gy(m2),
     $     a(n10,m2),b(m2,m1),gb(m1,m1),sav,chi2,vec(m2)
      double precision x(maxn1),x0(n10)
      save a,b,gb,x0,y
      data y0/10.2,  6.4, 22.1,  1.9, 32.2,  6.1,
     $       33.5, 26.5, 20.2, 33.3,  6.0, 19.2/
      data gy/ .25, 1.,    .25, 1.,    .25, 1.,
     $         .25, 1.,    .25, 1.,    .25, 1./
c    test problem
c      chi2=0.001
c      do 90 i=1,6
c        sav=2.*3.141592*i/6.+sin(1023+255*i-97.)
c        y0(2*i-1)=20.+12.*sin(sav)+chi2*sin(1023+255*i-17.)
c 90     y0(2*i)=18.+12.*cos(sav)+chi2*cos(1023.+255*i)
      if (iter.gt.1) then
c  Correct observed values y and compute chi2
        do 5 j=1,n
          sav=x(j)-x0(j)
          do 5 i=1,m1
 5          a(n1,i)=a(n1,i)-a(j,i)*sav
        do 51 i=1,m1
          sav=0.
          do 52 j=i,m1
 52         sav=sav+gb(i,j)*a(n1,j)
 51       a(n1,i)=sav          
        do 6 j=1,m2
          sav=0.
          do 7 i=1,m1
 7          sav=b(j,i)*a(n1,i)+sav
 6        y(j)=sav/sqrt(gy(j))+y(j)
        chi2=0.
        do 8 j=1,m1
          vec(j)=0.
          do 8 i=1,m2
 8          vec(j)=vec(j)+b(i,j)*(y(i)-y0(i))
        do 9 i=1,m1
          sav=0.
          do 91 j=i,m1
 91         sav=sav+gb(i,j)*vec(j)
 9        chi2=sav*sav+chi2
c   *****  This section defines the circle problem ******
      else
        n1=n10
        n=n1-1
        idf=m1-n
        x(1)=21.
        x(2)=17.
        x(3)=15.
        do 14 i=1,m2
 14       y(i)=y0(i)
      endif
c  Compute conditions -f(x,y) and derivatives a=df/dx and b=df/dy 
c  (x-unknowns, y-observations)
      do 11 l=1,m1
        do 12 j=1,m2
 12       b(j,l)=0.
        a(n1,l)=
     $    -((x(1)-y(2*l-1))**2.+(x(2)-y(2*l))**2.-x(3)*x(3))
        a(3,l)=-2.*x(3)
        do 13 j=1,2
          i=(l-1)*2+j
          a(j,l)=(x(j)-y(i))*2.
 13       b(i,l)=-a(j,l)
 11   continue
c  **** End of problem definition ****
c  Compute conditions weight matrix gb=(gy**(-1/2)*b)**(-2)
      do 16 i=1,m1+1
        do 15 l=1,m2
 15       vec(l)=b(l,i)/gy(l)
        do 16 j=1,i
          sav=0.
          if (i.le.m1) then
            do 17 l=1,m2
 17           sav=vec(l)*b(l,j)+sav
          endif
 16       eq(i,j)=sav
        eq(m1+1,m1+1)=1.
        if (cracow(eq,m1+1,maxn1,-1,det).le.0.) then
          write(*,*) 
     $      'Error or singular weight matrix gb'
        endif
c  Compute weighted condition matrix a=sqrt(gb)*a 
      do 25 i=1,m1
        do 26 j=1,n1
          sav=0.
          do 27 l=i,m1
 27         sav=eq(i,l)*a(j,l)+sav
 26       a(j,i)=sav
        do 25 j=i,m1
 25       gb(i,j)=eq(i,j)
c   Set normal equations eq=a*a
      do 1 i=1,n1
        x0(i)=x(i)
        do 1 j=1,i
          sav=0.
          do 2 l=1,m1
 2          sav=a(i,l)*a(j,l)+sav
 1        eq(i,j)=sav
      eq(n1,n1)=eq(n1,n1)+chi2
      end
